@@ -56,6 +56,7 @@ how the various predicates can be called from Prolog.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #define _CRT_SECURE_NO_WARNINGS 1
+#define _CRT_NONSTDC_NO_DEPRECATE 1
 #define PROLOG_MODULE "user"
 #include <iostream>
 #include <sstream>
@@ -114,6 +115,14 @@ PREDICATE(hello, 2)
   return A2.unify_string(buffer.str());
 }
 
+PREDICATE(helloex, 0)
+{ throw PlUnknownError("an exception");
+}
+
+PREDICATE(helloex, 1)
+{ throw PlUnknownError(A1);
+}
+
 PREDICATE(hello2, 2)
 { PlAtom atom_a1(A1.as_atom());
   std::stringstream buffer;
@@ -138,8 +147,8 @@ PREDICATE(hello3, 2)
   // character in it. In addition, a NUL ('\0') in the atom will cause
   // the rest of the atom to not be printed.
 
-  int len = Ssnprintf(buf, sizeof buf,
-		      "Hello3 %Ws\n", atom_a1.as_wstring().c_str());
+  int len = SsnprintfX(buf, sizeof buf,
+		       "Hello3 %Ws\n", atom_a1.as_wstring().c_str());
   if ( len >= 0 )
     // TODO: use len when fixed: https://github.com/SWI-Prolog/swipl-devel/issues/1074
     return A2.unify_chars(PL_STRING|REP_UTF8, strlen(buf), buf);
@@ -221,7 +230,7 @@ PREDICATE(list_modules, 1)
   PlTermv av(1);
 
   PlQuery q("current_module", av);
-  while( q.next_solution() )
+  while( PlWrap<int>(q.next_solution()) )
     buffer << av[0].as_string() << endl;
 
   q.cut();
@@ -239,7 +248,7 @@ PREDICATE(average, 3)			/* average(+Templ, :Goal, -Average) */
      PlQuery q("call", av);
   */
   PlQuery q("call", PlTermv(A2));
-  while( q.next_solution() )
+  while( PlWrap<int>(q.next_solution()) )
   { sum += A1.as_long();
     n++;
   }
@@ -249,7 +258,7 @@ PREDICATE(average, 3)			/* average(+Templ, :Goal, -Average) */
 
 PREDICATE(call_cpp, 2)
 { PlQuery q(A1.as_string(), PlTermv(A2));
-  PlCheckFail(q.next_solution());
+  PlCheckFail(PlWrap<int>(q.next_solution()));
   // There's no need for calling q.cut() - it's done implicitly by the
   // query's destructor.
   return true;
@@ -257,7 +266,7 @@ PREDICATE(call_cpp, 2)
 
 PREDICATE(call_cut, 1)
 { PlQuery q(A1.as_string(), PlTermv());
-  PlCheckFail(q.next_solution());
+  PlCheckFail(PlWrap<int>(q.next_solution()));
   q.cut(); // This tests that ~PlQuery() behaves correctly if cut() had been called
   return true;
 }
@@ -369,7 +378,7 @@ PREDICATE(if_then_ffi, 4)
   term_t t1 = A1.unwrap(), t2 = A2.unwrap();
   int t1_t2_unified = PL_unify(t1, t2);
   if ( !t1_t2_unified )
-  { if ( PL_exception(0) )
+  { if ( Plx_exception(0) )
       { PL_close_foreign_frame(fid);
         return FALSE;
       }
@@ -501,13 +510,16 @@ PREDICATE(cpp_call_, 3)
 
 PREDICATE(cpp_atom_codes, 2)
 { int rc = PlCall("atom_codes", PlTermv(A1, A2));
+  PlStream strm(Scurrent_output); // for debugging output
   if ( !rc )
-  { PlException ex(PlTerm(Plx_exception(0)));
-    PlStream strm(Scurrent_output);
+  { PlExceptionFromQid ex;
     if ( ex.is_null() )
       strm.printf("atom_codes failed\n");
     else
       strm.printf("atom_codes failed: ex: %s\n", ex.as_string().c_str()); // Shouldn't happen
+    throw ex;
+  } else
+  { strm.printf("atom_codes succeeded: %s -> %s\n", A1.as_string().c_str(), A2.as_string().c_str());
   }
   return rc;
 }
@@ -1401,7 +1413,36 @@ PREDICATE(cvt_i_bool, 2)
 
 // TODO: add tests for PL_cvt_i_*() (using PlTerm::integer())
 
-// TODO: add PlEngine tests
+int fake_main(int argc, char *argv[]) // TODO: add test that calls this.
+{ PlEngine e(argc, argv);
+  PlTermv av(1);
+  PlTerm_tail l(av[0]);
+
+  for(int i=0; i<argc; i++)
+    PlCheckFail(l.append(PlTerm_string(argv[i])));
+  PlCheckFail(l.close());
+
+  PlQuery q("entry", av);
+  try
+  { return PlWrap<int>(q.next_solution()) ? 0 : 1;
+  } catch ( PlException &ex )
+  { std::cerr << ex.what() << std::endl;
+    return 1;
+  }
+}
+
+int fake_main_legacy(int argc, char **argv) // TODO: add test that calls this.
+{ PlEngine e(argc, argv);
+  PlTermv av(1);
+  PlTerm_tail l(av[0]);
+
+  for(int i=0; i<argc; i++)
+    PlCheckFail(l.append(PlTerm_string(argv[i])));
+  PlCheckFail(l.close());
+
+  PlQuery q("entry", av);
+  return PlWrap<int>(q.next_solution()) ? 0 : 1;
+}
 
 PREDICATE(throw_domain_cpp0, 1)
 { return Plx_domain_error("footype", A1.unwrap());
@@ -1531,7 +1572,7 @@ struct MyConnection
 
 
 // The following code is taken from
-// pl2cpp2.doc \subsubsection{Sample PlBlob code}
+// pl2cpp.doc \subsubsection{Sample PlBlob code}
 // with some minor changes for testing
 
 struct MyBlob;
@@ -1551,7 +1592,7 @@ struct MyBlob : public PlBlob
       connection(std::make_unique<MyConnection>(connection_name)),
       blob_name(connection_name)
   { if ( !connection ) // make_unique should have thrown exception if it can't allocate
-      PL_api_error("MyBlob(%s) connection=%p", blob_name.c_str(), connection.get());
+      PL_api_error("MyBlob(%s) connection=%p", blob_name.c_str(), static_cast<const void*>(connection.get()));
     if ( !connection->open() )
       throw MyBlobError("my_blob_open_error");
     if ( name_contains("FAIL_new") ) // Test error handling
@@ -1650,7 +1691,7 @@ PREDICATE(portray_my_blob, 2)
 
 
 // TODO: Add the following code to
-// pl2cpp2.doc \subsubsection{Sample PlBlob code (pointer)}
+// pl2cpp.doc \subsubsection{Sample PlBlob code (pointer)}
 
 struct MyFileBlob;
 
@@ -1698,7 +1739,7 @@ struct MyFileBlob : public PlBlob
   { assert(sizeof buffer_[0] == sizeof (char));
     assert(sizeof (char) == 1);
 
-    buffer_.reserve(count);
+    buffer_.resize(count); // .reserve(count) gives an ASAN error
     return std::string(buffer_.data(),
                        std::fread(buffer_.data(), sizeof buffer_[0], count, file_));
   }
@@ -2174,7 +2215,7 @@ PREDICATE(free_blob, 1)
 
 PREDICATE(nil_repr, 1)
 { char buf[100];
-  snprintf(buf, sizeof buf, "%p", (void*) nullptr);
+  snprintf(buf, sizeof buf, "%p", static_cast<const void*>(nullptr));
   return A1.unify_string(buf);
 }
 
